<?php

/**
 * Class representing an audio-CD with CDDB information
 * 
 * @author Keith Palmer <Keith@UglySlug.com>
 * @category Net
 * @package Net_CDDB
 * @license http://www.opensource.org/licenses/bsd-license.php BSD License
 */

/**
 * Require CDDB class, we need the constants
 */
require_once 'Net/CDDB.php';

/**
 * Class representing an audio-CD with CDDB information
 * 
 * This class has methods to access all of the information a CDDB server has 
 * available about a specific audio CD. 
 * 
 * @package Net_CDDB
 */
class Net_CDDB_Disc
{
	/**
	 * Array of track titles
	 * 
	 * @var array
	 * @access protected
	 */
	var $_tracks;
	
	/**
	 * Array of extra data about tracks
	 * 
	 * @var array
	 * @access protected
	 */
	var $_extras;
	
	/**
	 * Array of track offsets ( in seconds )
	 * 
	 * @var array
	 * @access protected
	 */
	var $_offsets;
	
	/**
	 * Artist name
	 * 
	 * @var string
	 * @access protected
	 */
	var $_artist;
	
	/**
	 * Disc/audio CD title
	 * 
	 * @var string
	 * @access protected
	 */
	var $_title;
	
	/**
	 * 8-char discid field
	 * 
	 * @var string
	 * @access protected
	 */
	var $_discid;
	
	/**
	 * CDDB genre string
	 * 
	 * @var string
	 * @access protected
	 */
	var $_genre;
	
	/**
	 * Year audio CD was published
	 * 
	 * @var integer
	 * @access protected
	 */
	var $_year;
	
	/**
	 * Length in seconds of the audio-CD
	 * 
	 * @var integer
	 * @access protected
	 */
	var $_length;
	
	/**
	 * Playorder of tracks ( optional CDDB attribute )
	 * 
	 * @var string
	 * @access protected
	 */
	var $_playorder;
	
	/**
	 * CDDB revision attribute
	 * 
	 * @var integer
	 * @access protected
	 */
	var $_revision;
	
	/**
	 * Revision id if we've edited the record
	 * 
	 * @var integer
	 * @access protected
	 */
	var $_this_revision;
	
	/**
	 * Construct a new Net_CDDB_Disc object
	 * 
	 * The Net_CDDB_Disc actually supports two differnet methods of constructing 
	 * a Net_CDDB_Disc object. The first is to provide all of the parameters. 
	 * An alternative constructor allows passing an associative array as the 
	 * first *and only* parameter with the following keys: 
	 * 	- discid, 8-char discid
	 * 	- artist, string artist name
	 * 	- title, string title of the cd
	 * 	- genre, string genre of the cd
	 * 	- year, integer year the cd was published
	 * 	- tracks, array of track names
	 * 	- extras, array of extra data about each track (lyrics, subtitles, also know as, etc.)
	 * 	- offsets, array of track offsets (in seconds from beginning of disc)
	 * 	- length, length of disc (in seconds)
	 * 	- revision, CDDB record revision # of disc 
	 * 	- playorder, string describing custom track play order
	 * 
	 * @access public
	 *  
	 * @param string $arr_or_discid *See note above!*
	 * @param string $artist
	 * @param string $title
	 * @param string $genre
	 * @param integer $year
	 * @param array $tracks Title of each track
	 * @param array $extras Extra data about each track
	 * @param array $offsets Offset (in seconds) of each track
	 * @param integer $length
	 * @param integer $revision The CDDB revision from the CDDB record
	 * @param string $playorder Optional play-order of the tracks
	 */
	function Net_CDDB_Disc($arr_or_discid, $artist = '', $title = '', $genre = '', $year = '', $tracks = array(), $extras = array(), $offsets = array(), $length = 0, $revision = 0, $playorder = '')
	{
		if (is_array($arr_or_discid)) // If the first parameter is an array, we'll treat this as an associative array constructor
		{
			$defaults = array(
				'discid' => '', 
				'artist' => '', 
				'title' => '', 
				'genre' => '', 
				'year' => '', 
				'tracks' => array(), 
				'extras' => array(), 
				'offsets' => array(), 
				'length' => 0, 
				'revision' => 0, 
				'playorder' => '' );
			
			$arr_or_discid = array_merge($defaults, $arr_or_discid); // Ensure all required variables are set
			$arr_or_discid['arr_or_discid'] = $arr_or_discid['discid'];
			
			extract($arr_or_discid); // Extract into local scope so we can assign next
		}
				
		$this->_discid = $arr_or_discid;
		$this->_artist = $artist;
		$this->_title = $title;
		$this->_genre = $genre;
		$this->_year = $year;
		$this->_tracks = $tracks;
		$this->_extras = $extras;
		$this->_offsets = $offsets;
		$this->_length = $length;
		$this->_revision = $revision;
		$this->_playorder = $playorder;
		
		// Some checks to ensure good data when we use toString()
		for ($i = count($this->_extras); $i < count($this->_tracks); $i++) {
			$this->_extras[] = '';
		}
		
		for ($i = count($this->_tracks); $i < count($this->_extras); $i++) {
			$this->_tracks[] = '';
		}
		
		$this->_this_revision = -1;
	}
	
	/**
	 * Get the 8-char discid for the disc
	 * 
	 * @access public
	 * 
	 * @return string
	 */
	function getDiscId()
	{
		return $this->_discid;
	}
	
	/**
	 * Get the artist for the audio CD
	 * 
	 * @access public
	 * 
	 * @return string
	 */
	function getArtist()
	{
		return $this->_artist;
	}
	
	/**
	 * Get the title of this disc
	 * 
	 * @access public
	 * 
	 * @return string
	 */
	function getTitle()
	{
		return $this->_title;
	}
	
	/**
	 * Get the genre this disc belongs to
	 * 
	 * @access public
	 * 
	 * @return string
	 */
	function getGenre()
	{
		return $this->_genre;
	}
	
	/**
	 * Get the number of tracks the CD contains
	 * 
	 * @access public
	 * 
	 * @return integer
	 */
	function numTracks()
	{
		return count($this->_tracks);
	}
	
	/**
	 * Get the track title for a specific track number ( track #s start at 0 )
	 * 
	 * <code>
	 * print("The title of the first song is: ");
	 * $disc -> getTrackTitle(0);
	 * </code>
	 * 
	 * @param integer $track_num
	 * @return string
	 */
	function getTrackTitle($track_num)
	{
		if (isset($this->_tracks[$track_num])) {
			return $this->_tracks[$track_num];
		} else {
			return null;
		}
	}
	
	/**
	 * Get the track offset for a specific track
	 * 
	 * @param integer $track_num
	 * @return integer
	 */
	function getTrackOffset($track_num)
	{
		if (isset($this->_offsets[$track_num])) {
			return $this->_offsets[$track_num];
		} else {
			return null;
		}
	}
	
	/**
	 * Retrieve the length of this disc in seconds
	 * 
	 * @access public
	 * 
	 * @param bool $formatted Whether or not to return in string format: HH:MM:SS ( defaults to returning an integer number of seconds in length )
	 * @return integer
	 */
	function getDiscLength($formatted = false)
	{
		if ($formatted) {
			$hours = floor($this->_length / (60 * 60));
			$minutes = floor($this->_length % (60 * 60));
			$seconds = $this->_length % 60;
			
			return sprintf('%02d', $hours) . ':' . sprintf('%02d', $minutes) . ':' . sprintf('%02d', $seconds);
		} else {
			return $this->_length;
		}
	}
	
	/**
	 * Return a string representation of this CDDB Disc ( the CDDB file format )
	 * 
	 * @todo Make EXTD data field work
	 * 
	 * @access public
	 * 
	 * @return string
	 */
	function toString()
	{
		$str = "# xcmd\r\n";
		$str .= "#\r\n";
		$str .= "# Track frame offsets:\r\n";
		
		foreach ($this->_offsets as $key => $offset) {
			$str .= '#    ' . $offset . "\r\n";
		}
			
		$str .= "#\r\n";
		$str .= '# Disc length: ' . $this->_length . " seconds\r\n";
		$str .= "#\r\n";
		$str .= "# Revision: " . ($this->_revision + 1) . "\r\n";
		$str .= "# Submitted via: PHP/PEAR/Net_CDDB " . NET_CDDB_VERSION . "\r\n";
		$str .= "#\r\n";
		$str .= 'DISCID=' . $this->_discid . "\r\n";
		$str .= 'DTITLE=' . $this->_artist . ' / ' . $this->_title . "\r\n";
		$str .= 'DYEAR=' . $this->_year . "\r\n";
		$str .= 'DGENRE=' . $this->_genre . "\r\n";
		
		foreach ($this->_tracks as $key => $track) {
			$str .= 'TTITLE' . $key . '=' . $track . "\r\n";
		}
		
		$str .= 'EXTD=' . "\r\n";
		
		foreach ($this->_extras as $key => $extra) {
			$str .= 'EXTT' . $key . '=' . $extra . "\r\n";
		}
		
		$str .= 'PLAYORDER=' . $this->_playorder . "\r\n";
		
		return trim($str);
	}
}

?>