<?php

/**
 * Static class to calculate discids from track offsets/disc length
 * 
 * @author Keith Palmer <Keith@UglySlug.com>
 * @category Net
 * @package Net_CDDB
 * @license http://www.opensource.org/licenses/bsd-license.php BSD License
 */

/**
 * Static class providing helper functions to calculate disc ids
 * 
 * Although CDDB servers provide a method to calculate disc ids via sending a query
 * to the server and receiving the disc id in return, sometimes it's nice to be 
 * not have to query the server to get this information. Given the track offsets 
 * and disc length this static class can calculate the disc id. 
 * 
 * @package Net_CDDB
 */
class Net_CDDB_DiscId
{
	/**
	 * Calculate the disc id from the track offsets and disc length
	 * 
	 * @access public
	 * @static 
	 * 
	 * @uses Net_CDDB_DiscId::sum()
	 * 
	 * @param array $track_offsets
	 * @param int $disc_length Disc length in seconds
	 * @return string 8-char disc id
	 */
	function calculateDiscId($track_offsets, $disc_length)
	{
		$n = 0;
		
		$count = count($track_offsets);
		for ($i = 0; $i < $count; $i++) {
			$n = $n + Net_CDDB_DiscId::sum($track_offsets[$i] / 75);
		}
		
		// The $disc_length - 2 accounts for the 150 frame offset the RedBook standard uses... I think...
		return dechex(($n % 0xff) << 24 | ($disc_length - 2) << 8 | count($track_offsets));
	}
	
	/**
	 * Helper function for calculateDiscId() method, sum digits in track offset
	 * 
	 * @access public
	 * @static
	 * 
	 * @param int $n 
	 * @return int
	 */
	function sum($n)
	{
		$ret = 0;
		
		while ($n > 0) {
			$ret = $ret + ($n % 10);
			$n = (int) ($n / 10);
		}
		
		return $ret;	
	}
}

?>